﻿using System;
using System.Collections;
using BEPUphysics.CollisionTests.CollisionAlgorithms;
using BEPUphysics.CollisionTests.CollisionAlgorithms.GJK;
using BEPUphysics.Constraints;
using BEPUphysics.Materials;
using BEPUphysics.Settings;
using BEPUutilities.Threading;
using UnityEngine;
using PSpace = BEPUphysics.Space;
using PVector3 = BEPUutilities.Vector3;

public class PhysicsManager : Singleton<PhysicsManager>
{
    public Vector3 gravity = new Vector3(0f, -9.81f, 0f);

    private ParallelLooper parallelLooper;

    public PSpace Space { get; set; }

    private void Awake()
    {
        Init();
        //StartCoroutine(UpdateSpace());
    }

    private void Update()
    {
        Space.Update(Time.deltaTime);
    }

    /*IEnumerator UpdateSpace()
    {
        while (true)
        {
            yield return new WaitForSecondsRealtime(0.01f);

            Space.Update();
        }
    }*/

    private void Init()
    {
        parallelLooper = new ParallelLooper();

        if (Environment.ProcessorCount > 1)
        {
            for (int i = 0; i < Environment.ProcessorCount; i++)
            {
                parallelLooper.AddThread();
            }
        }

        Space = new PSpace();
        Space.ForceUpdater.Gravity = new PVector3(gravity.x, gravity.y, gravity.z);

        MaterialManager.MaterialBlender = delegate (BEPUphysics.Materials.Material a,
            BEPUphysics.Materials.Material b, out InteractionProperties properties)
        {
            properties.KineticFriction = BEPUutilities.MathHelper.Max(a.KineticFriction, b.KineticFriction);
            properties.StaticFriction = BEPUutilities.MathHelper.Max(a.StaticFriction, b.StaticFriction);
            properties.Bounciness = BEPUutilities.MathHelper.Min(a.Bounciness, b.Bounciness);
        };

        ApplySettings();
    }

    private void ApplySettings()
    {
        MotionSettings.DefaultPositionUpdateMode = BEPUphysics.PositionUpdating.PositionUpdateMode.Discrete;
        SolverSettings.DefaultMinimumIterationCount = 1;
        Space.Solver.IterationLimit = 10;
        GeneralConvexPairTester.UseSimplexCaching = false;
        MotionSettings.UseExtraExpansionForContinuousBoundingBoxes = false;

        // Set all the scaling settings back to their defaults.
        Space.DeactivationManager.VelocityLowerLimit = .5f;
        CollisionResponseSettings.MaximumPenetrationRecoverySpeed = 2;
        CollisionResponseSettings.BouncinessVelocityThreshold = 1;
        CollisionResponseSettings.StaticFrictionVelocityThreshold = .2f;
        CollisionDetectionSettings.ContactInvalidationLength = .1f;
        CollisionDetectionSettings.ContactMinimumSeparationDistance = .03f;
        CollisionDetectionSettings.MaximumContactDistance = .1f;
        CollisionDetectionSettings.DefaultMargin = .04f;
        CollisionDetectionSettings.AllowedPenetration = .01f;
        SolverSettings.DefaultMinimumImpulse = .001f;

        // Adjust epsilons back to defaults
        BEPUutilities.Toolbox.Epsilon = 1e-7f;
        BEPUutilities.Toolbox.BigEpsilon = 1e-5f;
        MPRToolbox.DepthRefinementEpsilon = 1e-4f;
        MPRToolbox.RayCastSurfaceEpsilon = 1e-9f;
        MPRToolbox.SurfaceEpsilon = 1e-7f;
        PairSimplex.DistanceConvergenceEpsilon = 1e-7f;
        PairSimplex.ProgressionEpsilon = 1e-8f;

        //Space.TimeStepSettings.TimeStepDuration = .01m;
    }
}
