﻿using BEPUphysics.Entities;
using UnityEngine;

public abstract class PhysicsShape : MonoBehaviour
{
    public abstract Entity CreateEntity(float mass);

    public Vector3 center;
    public PhysicMaterial material;

    private Vector3 tPos;
    private Quaternion tRot;

    public Entity Entity { get; private set; }

    private void Start()
    {
        var pb = GetComponent<PhysicsBody>();
        Entity = CreateEntity(pb != null ? pb.mass : 0f);
        PhysicsManager.Instance.Space.Add(Entity);

        var tRot = transform.rotation;
        Entity.Orientation = new BEPUutilities.Quaternion
        {
            X = tRot.x,
            Y = tRot.y,
            Z = tRot.z,
            W = tRot.w
        };

        Entity.Material.KineticFriction = material.dynamicFriction;
        Entity.Material.StaticFriction = material.staticFriction;
        Entity.Material.Bounciness = material.bounciness;

        if (pb != null)
        {
            Entity.LinearDamping = pb.damping;
            Entity.AngularDamping = pb.angularDamping;
        }
    }

    private void Update()
    {
        var ePos = Entity.Position;
        tPos.Set((float)ePos.X, (float)ePos.Y, (float)ePos.Z);
        var eRot = Entity.Orientation;
        tRot.Set((float)eRot.X, (float)eRot.Y, (float)eRot.Z, (float)eRot.W);
        transform.position = tPos;
        transform.rotation = tRot;
    }
}
